﻿/* ============================================================
* Author:       xieqin
* Time:         2014/11/26 15:45:52
* FileName:     env
* Purpose:      env存放全局数据
* =============================================================*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Config;
using Protocols;
using Log;
using System.Diagnostics;
using System.Threading;
using System.Collections.Concurrent;

namespace ServerLib
{
    /// <summary>
    /// 把await后代码切回主线程调用，就不需要考虑线程同步了
    /// </summary>
    public sealed class SingleThreadSynchronizationContext : SynchronizationContext
    {
        private readonly ConcurrentQueue<KeyValuePair<SendOrPostCallback, object>> m_queue = new ConcurrentQueue<KeyValuePair<SendOrPostCallback, object>>();

        public override void Post(SendOrPostCallback d, object state)
        {
            m_queue.Enqueue(new KeyValuePair<SendOrPostCallback, object>(d, state));
        }

        public override void Send(SendOrPostCallback d, object state)
        {
            base.Send(d, state);
        }

        /// <summary>
        /// 供主线程调用
        /// </summary>
        public void RunOnCurrentThread()
        {
            KeyValuePair<SendOrPostCallback, object> workItem;
            List<KeyValuePair<SendOrPostCallback, object>> lstItems = new List<KeyValuePair<SendOrPostCallback, object>>();

            int count = m_queue.Count;
            for (int i = 0; i < count; i++)
            {
                if (m_queue.TryDequeue(out workItem))
                {
                    lstItems.Add(workItem);
                }
            }

            foreach (var item in lstItems)
            {
                item.Key(item.Value);
            }
        }
    }
}
